import AST
import globals
import utils
import Visitor

def filename_is_normalized(filename):
    return utils.abspath(filename) == filename and '\\' not in filename

class ASTChecker(Visitor.Visitor):
    def __init__(self):
        self.root_node_seen = False

    def visit_BlockRule(self, astnode, arg):
        assert astnode.rules
        assert filename_is_normalized(astnode.rootdir)

        if self.root_node_seen:
            assert astnode.parent
        if astnode.parent is None:
            self.root_node_seen = True

        for rule in astnode.rules:
            assert rule.parent == astnode
            assert isinstance(rule, AST.Rule)
            rule.visit(self)

        astnode.innerdir_common_prefix()

    def visit_IfBlockRule(self, astnode, arg):
        assert astnode.rootdir == astnode.parent.rootdir
        assert filename_is_normalized(astnode.rootdir)
        assert astnode.if_expr
        assert astnode.if_expr.expr_type == globals.E_BOOLEAN
        assert astnode.if_expr.parent == astnode
        self.visit_BlockRule(astnode, arg)

    def visit_GloblistRule(self, astnode, arg):
        assert astnode.rootdir == astnode.parent.rootdir
        assert filename_is_normalized(astnode.rootdir)
        assert astnode.glob_patterns
        assert astnode.inclusive is not None
        for pattern in astnode.glob_patterns:
            assert pattern.parent == astnode
            assert isinstance(pattern, AST.GlobPattern)
            pattern.visit(self)
        if astnode.if_expr:
            astnode.if_expr.visit(self)

    def visit_ForEachRule(self, astnode, arg):
        assert astnode.rootdir == astnode.parent.rootdir
        assert filename_is_normalized(astnode.rootdir)
        assert astnode.variable_expr
        assert astnode.variable_expr.variable_name
        assert astnode.variable_expr.expr_type
        assert astnode.glob_patterns
        assert astnode.if_expr
        assert astnode.inclusive is not None

        astnode.variable_expr.visit(self)
        for pattern in astnode.glob_patterns:
            assert pattern.parent == astnode
            assert isinstance(pattern, AST.GlobPattern)
            pattern.visit(self)

        assert isinstance(astnode.if_expr, AST.Expression)
        astnode.if_expr.visit(self)

    def visit_GlobPattern(self, astnode, arg):
        assert astnode.parent
        assert astnode.pattern
        assert '\\' not in astnode.pattern
        assert astnode.rootdir == astnode.parent.rootdir
        assert filename_is_normalized(astnode.rootdir)
        assert astnode.recursive is not None
        assert astnode.fullglob
        assert filename_is_normalized(astnode.fullglob)
        assert filename_is_normalized(astnode.innerdir)

    def visit_AndExpression(self, astnode, arg):
        assert astnode.rootdir == astnode.parent.rootdir
        assert filename_is_normalized(astnode.rootdir)
        assert astnode.expr_type is not None
        assert astnode.expr1.parent == astnode
        assert astnode.expr2.parent == astnode
        assert astnode.expr1.expr_type == globals.E_BOOLEAN
        assert astnode.expr2.expr_type == globals.E_BOOLEAN
        astnode.expr1.visit(self)
        astnode.expr2.visit(self)

    def visit_OrExpression(self, astnode, arg):
        assert astnode.rootdir == astnode.parent.rootdir
        assert filename_is_normalized(astnode.rootdir)
        assert astnode.expr_type is not None
        assert astnode.expr1.parent == astnode
        assert astnode.expr2.parent == astnode
        assert astnode.expr1.expr_type == globals.E_BOOLEAN
        assert astnode.expr2.expr_type == globals.E_BOOLEAN
        astnode.expr1.visit(self)
        astnode.expr2.visit(self)

    def visit_NotExpression(self, astnode, arg):
        assert astnode.rootdir == astnode.parent.rootdir
        assert filename_is_normalized(astnode.rootdir)
        assert astnode.expr_type is not None
        assert astnode.expr.expr_type == globals.E_BOOLEAN
        astnode.expr.visit(self)

    def visit_FunctionExpression(self, astnode, arg):
        assert astnode.rootdir == astnode.parent.rootdir
        assert filename_is_normalized(astnode.rootdir)
        assert astnode.expr_type is not None
        for param in astnode.params:
            param.visit(self)
            assert param.parent == astnode
            assert isinstance(param, AST.Expression)

    def visit_CompareExpression(self, astnode, arg):
        assert astnode.rootdir == astnode.parent.rootdir
        assert filename_is_normalized(astnode.rootdir)
        assert astnode.expr_type == globals.E_BOOLEAN
        astnode.expr1.visit(self)
        astnode.expr2.visit(self)
        assert astnode.expr1.parent == astnode
        assert astnode.expr2.parent == astnode

    def visit_LiteralExpression(self, astnode, arg):
        assert astnode.rootdir == astnode.parent.rootdir
        assert filename_is_normalized(astnode.rootdir)
        assert astnode.expr_type is not None

    def visit_VariableExpression(self, astnode, arg):
        assert astnode.rootdir == astnode.parent.rootdir
        assert filename_is_normalized(astnode.rootdir)
        assert astnode.expr_type is not None
        assert astnode.variable_obj is not None
