import sys
from distutils.core import setup

LONG_DESC = '''\
File Selection Language (FSL) is a descriptive language for file selection.
It is used to selectively pick files from a directory structure.
FSL is useful for selective backups, for instance. FSL uses glob patterns
as the basic building block. For fine-tuning the selection,
inclusion/exclusion rule combinations and conditional expressions are
available. File size and modification date can be used in expressions.'''

MANUAL_INSTALL_INSTRUCTIONS = '''\
- If you plan to use FSL from the command line interface only,
  copy the FSL file tree into a directory of your choosing.
- If you plan to use FSL as a Python module as well,
  copy the fsl/ directory somewhere on your Python search
  path. To make sure you installed FSL correctly on the search
  path, execute "import fsl.fsltool" in a Python session.'''

TEST_DIR = 'testdir/'

TEST_DIR_STRUCT = [
    TEST_DIR + "and",
    TEST_DIR + "filea",
    TEST_DIR + "fileb.txt",
    TEST_DIR + "filec.txt.exe",
    TEST_DIR + "filed",
    TEST_DIR + "ALLCAPS",
    TEST_DIR + ".hidden",
    TEST_DIR + "long-filename-1234567890-abcdefghijkl",
    TEST_DIR + "dir1/filea",
    TEST_DIR + "dir1/filea-a-a",
    TEST_DIR + "dir1/dir2/asdf",
    TEST_DIR + "dir1/dir2/dir3/filez",
    TEST_DIR + "dir1/dir2/dir3/qwerty",
    TEST_DIR + "dir xyz/aaa",
    TEST_DIR + "dir xyz/filea",
    TEST_DIR + "dir xyz/q q q",
]

CLASSIFIERS = [
          'Development Status :: 4 - Beta',
          'Environment :: Console',
          'Intended Audience :: System Administrators',
          'License :: OSI Approved :: BSD License',
          'Operating System :: Microsoft :: Windows',
          'Operating System :: POSIX',
          'Programming Language :: Python',
          'Topic :: System :: Archiving',
]

package_data = {
    'fsl.test': ['fsltool_files/*', 'interpreter_files/*', 'lexer_files/*',
        'make_struct.sh', 'struct.txt'] + TEST_DIR_STRUCT
}

if __name__=='__main__':
    major, minor, micro, rellevel, serial = sys.version_info
    if (major, minor) < (2, 4):
        print 'Automated setup works only with Python 2.4 or greater.'
        print 'With former versions, you must install manually.'
        print 'Install instructions:'
        print MANUAL_INSTALL_INSTRUCTIONS
        sys.exit(1)

    setup(name='fsl',
          version='0.5',
          description='File Selection Language',
          author='Kristian Ovaska',
          author_email='kristian.ovaska [at] helsinki.fi',
          url='http://www.cs.helsinki.fi/u/hkovaska/fsl/',
          packages=['fsl', 'fsl.test'],
          package_data=package_data,
          classifiers=CLASSIFIERS,
          long_description = LONG_DESC,
          license='BSD'
         )
